//====================================================== file = common.c ======
//=  Common functions for REMOTE master and remote                            =
//=============================================================================
//=  Notes: 1) Contains winsock stuff for master and remote communications    =
//=---------------------------------------------------------------------------=
//=  Contains: get_addr() - Get address of a hostname                         =
//=            get_name() - Get hostname of an address                        =
//=            recv_file() - Receive a file                                   =
//=            send_file() - Send a file                                      =
//=            recv_command() - Receive a comman                              =
//=            send_command() - Send a command                                =
//=            get_time() - Get and return urrent date and time               =
//=---------------------------------------------------------------------------=
//=  History:  KJC (09/19/01) - Genesis                                       =
//=============================================================================

//----- Include files ---------------------------------------------------------
#include "common.h"            // Needed for common stuff

//=============================================================================
//=  Resolve ip_addr from input host_name                                     =
//=   - Assumes that ip_addr is allocated in caller                           =
//=============================================================================
int get_addr(char *host_name, char *ip_addr)
{
  WORD wVersionRequested = MAKEWORD(1,1);  // WSA functions
  WSADATA wsaData;                         // WSA functions
  struct hostent *host;                    // Structure for gethostbyname()
  struct in_addr  address;                 // Structure for Internet address
  int             retcode;                 // Return code

  // Initialize winsock
  WSAStartup(wVersionRequested, &wsaData);

  // Do a gethostbyname() on host_name
  host = gethostbyname(host_name);

  // Copy address in host structure to ip_addr
  if (host == NULL)
  {
    printf("*** ERROR in get_addr() - IP address not found for host %s \n",
      host_name);
    return(-1);
  }
  else
  {
    memcpy(&address, host->h_addr, 4);
    strcpy(ip_addr, inet_ntoa(address));
  }

  // Clean-up and return
  WSACleanup();
  return(1);
}

//=============================================================================
//=  Resolve host_name form input ip_addr                                     =
//=   - Assumes that host_name is allocated in caller                         =
//=============================================================================
int get_name(char *ip_addr, char *host_name)
{
  WORD wVersionRequested = MAKEWORD(1,1);  // WSA functions
  WSADATA wsaData;                         // WSA functions
  struct hostent *host;                    // Structure for gethostbyname()
  struct in_addr  *address;                // Structure for Internet address
  int             retcode;                 // Return code

  // Initialize winsock
  WSAStartup(wVersionRequested, &wsaData);

  // Set-up structure
  address = (struct in_addr*) malloc(sizeof(struct in_addr));
  address->s_addr = inet_addr(ip_addr);

  // Do a gethostbyaddr() on ip_addr
  host = gethostbyaddr((char *) address, 4 , AF_INET);

  // Output host name if host found
  if (host == NULL)
  {
    printf("*** ERROR in get_name() - host name not found for IP addr %s \n",
      ip_addr);
    return(-1);
  }
  else
    strcpy(host_name, host->h_name);

  // Clean-up and return
  WSACleanup();
  return(1);
}

//=============================================================================
//=  Receive a file                                                           =
//=   - Assumes that command and host_name are allocated in caller            =
//=============================================================================
int recv_file(char *recv_file, char *host_name, int port_num)
{
  WORD wVersionRequested = MAKEWORD(1,1);  // WSA functions
  WSADATA wsaData;                         // WSA functions
  struct linger        ling;               // Socket linger structure
  int                  client_s;           // Client socket descriptor
  struct sockaddr_in   client_addr;        // Client Internet address
  struct in_addr       client_ip_addr;     // Client IP address
  int                  server_s;           // Server socket descriptor
  struct sockaddr_in   server_addr;        // Server Internet address
  int                  addr_len;           // Internet address length
  unsigned char        bin_buf[BUF_SIZE];  // File transfer buffer
  int                  length;             // Length of buffers transferred
  int                  fh;                 // File handle
  int                  i;                  // Loop counter
  int                  retcode;            // Return code

  // Initialize winsock, create a socket, and bind it
  WSAStartup(wVersionRequested, &wsaData);
  server_s = socket(AF_INET, SOCK_STREAM, 0);
  server_addr.sin_family = AF_INET;
  server_addr.sin_port = htons(port_num);
  server_addr.sin_addr.s_addr = htonl(INADDR_ANY);
  retcode = bind(server_s, (struct sockaddr *)&server_addr,
                 sizeof(server_addr));
  if (retcode < 0)
  {
     printf("*** ERROR in recv_file() - bind() failed (%d) \n",
       WSAGetLastError());
     return(-1);
  }

  // Listen and then accept a connection from the client
  listen(server_s, 1);
  addr_len = sizeof(client_addr);
  client_s = accept(server_s, (struct sockaddr *)&client_addr, &addr_len);
  if (client_s < 0)
  {
     printf("*** ERROR in recv_file() - accept() failed (%d) \n",
       WSAGetLastError());
     return(-1);
  }

  // Get the host_name of the connected client
  memcpy(&client_ip_addr, &client_addr.sin_addr.s_addr, 4);
  get_name(inet_ntoa(client_ip_addr), host_name);

  // Set socket options
  ling.l_onoff = 1;
  ling.l_linger = 1;
  setsockopt(server_s, SOL_SOCKET, SO_LINGER, (LPSTR) &ling, sizeof(ling));
  setsockopt(client_s, SOL_SOCKET, SO_LINGER, (LPSTR) &ling, sizeof(ling));

  // Receive the filename from the client
  retcode = recv(client_s, bin_buf, BUF_SIZE, 0);
  if (retcode < 0)
  {
     printf("*** ERROR in recv_file() - recv() failed (%d) \n",
       WSAGetLastError());
     return(-1);
  }
  strcpy(recv_file, bin_buf);

  // Do a security check on the filename (no slashes allowed)
  for (i=0; i<((int) strlen(recv_file)); i++)
    if (recv_file[i] == '\\')
    {
      printf("*** ERROR in recv_file() - security check failed \n");
      return(-1);
    }

  // Open the file to receive
  fh = open(recv_file, O_WRONLY | O_CREAT | O_TRUNC | O_BINARY,
            S_IREAD | S_IWRITE);
  if (fh == -1)
  {
    printf("*** ERROR in recv_file() - unable to open receive file %s \n",
      recv_file);
    return(-1);
  }

  // Receive the file from the client
  length = BUF_SIZE;
  while (length > 0)
  {
     length = recv(client_s, bin_buf, BUF_SIZE, 0);
     if (length < 0)
     {
       printf("*** ERROR in recv_file() - recv() failed (%d) \n",
         WSAGetLastError());
       return(-1);
     }
     write(fh, bin_buf, length);
  }

  // Close, clean-up, and return
  close(fh);
  closesocket(client_s);
  closesocket(server_s);
  WSACleanup();
  return(1);
}

//=============================================================================
//=  Send a file                                                              =
//=   - Assumes that send_file is allocated in caller                         =
//=============================================================================
int send_file(char *send_file, char *host_name, int port_num)
{
  WORD wVersionRequested = MAKEWORD(1,1);  // WSA functions
  WSADATA wsaData;                         // WSA functions
  struct linger        ling;               // Socket linger structure
  struct hostent       *host;              // Structure for gethostbyname()
  struct in_addr       addr;               // Structure for IP address
  char                 server_ip_addr[16]; // String for server IP address
  int                  server_s;           // Server socket descriptor
  struct sockaddr_in   server_addr;        // Server Internet address
  int                  addr_len;           // Internet address length
  unsigned char        bin_buf[BUF_SIZE];  // File transfer buffer
  int                  length;             // Length of buffers transferred
  int                  fh;                 // File handle
  int                  retcode;            // Return code

  // Initialize winsock
  WSAStartup(wVersionRequested, &wsaData);

  // Get IP address from host_name and copy into server_ip_addr
  host = gethostbyname(host_name);
  if (host == NULL)
  {
    printf("*** ERROR in send_file() - IP address not found for host %s \n",
      host_name);
    return(-1);
  }
  memcpy(&addr, host->h_addr, 4);
  memcpy(server_ip_addr, inet_ntoa(addr), 16);

  // Create a socket and then connect
  server_s = socket(AF_INET, SOCK_STREAM, 0);
  server_addr.sin_family = AF_INET;
  server_addr.sin_port = htons(port_num);
  server_addr.sin_addr.s_addr = inet_addr(server_ip_addr);
  retcode = connect(server_s, (struct sockaddr *)&server_addr,
                    sizeof(server_addr));
  if (retcode < 0)
  {
    printf("*** ERROR in send_command() - failed connect to host %s (%d) \n",
      host_name, WSAGetLastError());
    return(-1);
  }

  // Set socket options
  ling.l_onoff = 1;
  ling.l_linger = 1;
  setsockopt(server_s, SOL_SOCKET, SO_LINGER, (LPSTR) &ling, sizeof(ling));

  // Send the file name to the server
  strcpy(bin_buf, send_file);
  retcode = send(server_s, bin_buf, strlen(send_file) + 1, 0);
  if (retcode < 0)
  {
    printf("*** ERROR in send_file() - send() failed (%d) \n",
       WSAGetLastError());
    return(-1);
  }

  // Open the file to send
  if((fh = open(send_file, O_RDONLY | O_BINARY, S_IREAD | S_IWRITE)) == -1)
  {
     printf("*** ERROR - unable to open send file %s \n", send_file);
     return(-1);
  }

  // Sleep to allow server time to create the file
  Sleep(TURN_DELAY);

  // Send the file
  while(!eof(fh))
  {
     length = read(fh, bin_buf, BUF_SIZE);
     send(server_s, bin_buf, length, 0);
     if (length < 0)
     {
       printf("*** ERROR in send_file() - send() failed (%d) \n",
         WSAGetLastError());
       return(-1);
     }
  }

  // Give server time to receive
  Sleep(TURN_DELAY);

  // Close, clean-up, and return
  close(fh);
  closesocket(server_s);
  WSACleanup();
  return(1);
}

//=============================================================================
//=  Receive a command                                                        =
//=   - Assumes that command and host_name are allocated in caller            =
//=============================================================================
int recv_command(char *command, char *host_name, int port_num)
{
  WORD wVersionRequested = MAKEWORD(1,1);  // WSA functions
  WSADATA wsaData;                         // WSA functions
  struct linger        ling;               // Socket linger structure
  int                  client_s;           // Client socket descriptor
  struct sockaddr_in   client_addr;        // Client Internet address
  struct in_addr       client_ip_addr;     // Client IP address
  int                  server_s;           // Server socket descriptor
  struct sockaddr_in   server_addr;        // Server Internet address
  int                  addr_len;           // Internet address length
  unsigned char        bin_buf[BUF_SIZE];  // File transfer buffer
  int                  retcode;            // Return code

  // Initialize winsock, create a socket, and bind it
  WSAStartup(wVersionRequested, &wsaData);
  server_s = socket(AF_INET, SOCK_STREAM, 0);
  server_addr.sin_family = AF_INET;
  server_addr.sin_port = htons(port_num);
  server_addr.sin_addr.s_addr = htonl(INADDR_ANY);
  retcode = bind(server_s, (struct sockaddr *)&server_addr,
                 sizeof(server_addr));
  if (retcode < 0)
  {
     printf("*** ERROR in recv_command() - bind() failed (%d) \n",
       WSAGetLastError());
     return(-1);
  }

  // Listen and then accept a connection from the client
  listen(server_s, 1);
  addr_len = sizeof(client_addr);
  client_s = accept(server_s, (struct sockaddr *)&client_addr, &addr_len);
  if (client_s < 0)
  {
     printf("*** ERROR in recv_command() - accept() failed (%d) \n",
       WSAGetLastError());
     return(-1);
  }

  // Get the host_name of the connected client
  memcpy(&client_ip_addr, &client_addr.sin_addr.s_addr, 4);
  get_name(inet_ntoa(client_ip_addr), host_name);

  // Set socket options
  ling.l_onoff = 1;
  ling.l_linger = 1;
  setsockopt(client_s, SOL_SOCKET, SO_LINGER, (LPSTR) &ling, sizeof(ling));
  setsockopt(server_s, SOL_SOCKET, SO_LINGER, (LPSTR) &ling, sizeof(ling));

  // Receive the command from the client
  retcode = recv(client_s, bin_buf, BUF_SIZE, 0);
  if (retcode < 0)
  {
     printf("*** ERROR in recv_command() - recv() failed (%d) \n",
       WSAGetLastError());
     return(-1);
  }
  strcpy(command, bin_buf);

  // Close, clean-up, and return
  closesocket(client_s);
  closesocket(server_s);
  WSACleanup();
  return(1);
}

//=============================================================================
//=  Send a command                                                           =
//=   - Assumes that command is allocated in caller                           =
//=============================================================================
int send_command(char *command, char *host_name, int port_num)
{
  WORD wVersionRequested = MAKEWORD(1,1);  // WSA functions
  WSADATA wsaData;                         // WSA functions
  struct linger        ling;               // Socket linger structure
  struct hostent       *host;              // Structure for gethostbyname()
  struct in_addr       addr;               // Structure for IP address
  int                  addr_len;           // IP address length
  char                 server_ip_addr[16]; // String for server IP address
  int                  server_s;           // Server socket descriptor
  struct sockaddr_in   server_addr;        // Server IP address
  unsigned char        bin_buf[BUF_SIZE];  // File transfer buffer
  int                  length;             // Length of buffers transferred
  int                  retcode;            // Return code

  // Initialize winsock
  WSAStartup(wVersionRequested, &wsaData);

  // Get IP address from host_name and copy into server_ip_addr
  host = gethostbyname(host_name);
  if (host == NULL)
  {
    printf("*** ERROR in send_command() - IP address not found for host %s \n",
      host_name);
    return(-1);
  }
  memcpy(&addr, host->h_addr, 4);
  memcpy(server_ip_addr, inet_ntoa(addr), 16);

  // Create a socket and then connect to server
  server_s = socket(AF_INET, SOCK_STREAM, 0);
  server_addr.sin_family = AF_INET;
  server_addr.sin_port = htons(port_num);
  server_addr.sin_addr.s_addr = inet_addr(server_ip_addr);
  retcode = connect(server_s, (struct sockaddr *)&server_addr,
                    sizeof(server_addr));
  if (retcode < 0)
  {
    printf("*** ERROR in send_command() - failed connect to host %s (%d) \n",
      host_name, WSAGetLastError());
    return(-1);
  }

  // Set socket options for LINGER and REUSE
  ling.l_onoff = 1;
  ling.l_linger = 1;
  setsockopt(server_s, SOL_SOCKET, SO_LINGER, (LPSTR) &ling, sizeof(ling));

  // Copy command to bin_buf
  strcpy(bin_buf, command);

  // Send command to server
  retcode = send(server_s, bin_buf, strlen(command) + 1, 0);
  if (retcode < 0)
  {
    printf("*** ERROR in send_command() - send() failed (%d) \n",
       WSAGetLastError());
    return(-1);
  }

  // Give server time to receive
  Sleep(TURN_DELAY);

  // Close, clean-up, and return
  closesocket(server_s);
  WSACleanup();
  return(1);
}

//=============================================================================
//=  Get current date and time                                                =
//=   - Assumes that time_val is allocated in caller                          =
//=============================================================================
int get_time(char *time_val)
{
  time_t               timer;              // Timer variable
  struct tm            *tblock;            // Time stucture

  // Get the time, format it, and remove ending '\n' character
  timer = time(NULL);
  tblock = localtime(&timer);
  strcpy(time_val, asctime(tblock));
  time_val[strlen(time_val) - 1] = '\0';

  return(1);
}
