//========================================================= file = mm1.c =====
//=  A CSIM18 simulation of an M/M/1 queueing system                         =
//=   - Command line arguments are input file name and output file name      =
//============================================================================
//=  Notes: This is the basic model to demonstrate the REMOTE tool           =
//=--------------------------------------------------------------------------=
//=  Build: standard CSIM build                                              =
//=--------------------------------------------------------------------------=
//=  Execute: "mm1 infile.txt outfile.txt" where infile.txt contains three   =
//=           double values (lambda, mu, and simulation time).  Output is    =
//=           to outfile.txt and status.txt.  Sample input.txt file is:      =
//=             0.5, 1.0, 1000000.0                                          =
//=--------------------------------------------------------------------------=
//=  History:   KJC (09/19/01) - Genesis                                     =
//============================================================================

//----- Includes --------------------------------------------------------------
#include <stdio.h>               // Needed for fprintf() and fscanf()
#include <stdlib.h>              // Needed for atof()
#include "csim.h"                // Needed for CSIM18 stuff

//----- Globals ---------------------------------------------------------------
FACILITY Server;                 // Pointer for Server facility
double   Lambda;                 // Mean arrival rate (cust/sec)
double   Mu;                     // Mean service rate (cust/sec)

//----- Prototypes ------------------------------------------------------------
void generate(void);             // Generate Poisson customers
void queue1(void);               // Single server exponential service queue
void run_sim(double run_time);   // Status reporter

//=============================================================================
//==  Main program                                                           ==
//=============================================================================
void sim(int argc, char *argv[])
{
  FILE     *fp_in;               // Pointer to input file
  FILE     *fp_out;              // Pointer to output file
  double   sim_time;             // Simulation time in seconds
  char     instring[256];        // Input string

  // Check if sufficient command line arguments
  if (argc != 3)
  {
    printf("Need input file name and output file name as arguments \n");
    exit(1);
  }

  // Create the simulation
  create("sim");

  // Initialize CSIM stuff
  Server = facility("Server");

  // Open input file
  fp_in = fopen(argv[1], "r");
  if (fp_in == NULL)
  {
    printf("*** ERROR in opening input file %s \n", argv[1]);
    exit(1);
  }

  // Open output file
  fp_out = fopen(argv[2], "w");
  if (fp_out == NULL)
  {
    printf("*** ERROR in opening output file %s \n", argv[2]);
    exit(1);
  }

  // Read-in input parameters from input file
  fscanf(fp_in, "%s", instring);
  Lambda = atof(instring);
  fscanf(fp_in, "%s", instring);
  Mu = atof(instring);
  fscanf(fp_in, "%s", instring);
  sim_time = atof(instring);

  // Initiate generate function
  generate();

  // Run for sim_time using status generating run_sim()
  run_sim(sim_time);

  // Output results to output file
  fprintf(fp_out, "====================================================== \n");
  fprintf(fp_out, "== *** CSIM18 M/M/1 queueing system simulation ***  == \n");
  fprintf(fp_out, "====================================================== \n");
  fprintf(fp_out, "= Lambda             = %6.3f cust/sec \n", Lambda);
  fprintf(fp_out, "= Mu                 = %6.3f cust/sec \n", Mu);
  fprintf(fp_out, "= Simulation time    = %6.3f sec      \n", sim_time);
  fprintf(fp_out, "=----------------------------------------------------- \n");
  fprintf(fp_out, "= >>> Simulation results                             - \n");
  fprintf(fp_out, "=----------------------------------------------------- \n");
  fprintf(fp_out, "= Total CPU time     = %6.3f sec      \n", cputime());
  fprintf(fp_out, "= Utilization        = %6.3f %%       \n",
    100.0 * util(Server));
  fprintf(fp_out, "= Mean num in system = %6.3f cust     \n", qlen(Server));
  fprintf(fp_out, "= Mean response time = %6.3f sec      \n", resp(Server));
  fprintf(fp_out, "= Mean service time  = %6.3f sec      \n", serv(Server));
  fprintf(fp_out, "= Mean throughput    = %6.3f cust/sec \n", tput(Server));
  fprintf(fp_out, "====================================================== \n");

  // Close input and output files
  fclose(fp_in);
  fclose(fp_out);
}

//=============================================================================
//==  Process to generate Poisson customers                                  ==
//=============================================================================
void generate(void)
{
  create("generate");

  // Loop forever to create Poisson customers
  while(1)
  {
    hold(expntl(1.0 / Lambda));
    queue1();
  }
}

//=============================================================================
//==  Process for single server queue                                        ==
//=============================================================================
void queue1(void)
{
  create("queue1");

  // Reserve, hold with exponential service time, and release server
  reserve(Server);
  hold(expntl(1.0 / Mu));
  release(Server);
}

//=============================================================================
//==  Process to output run status to status.txt file                        ==
//=============================================================================
void run_sim(double run_time)
{
  FILE     *fp_status;           // Pointer to status output file
  int      num_inc;              // Number of increments
  int      i;                    // Loop counter

  // Open status.txt for writing status updates
  fp_status = fopen("status.txt", "a+");
  if (fp_status == NULL)
  {
    printf("*** ERROR in opening status file 'status.txt' \n");
    exit(1);
  }

  // Initialize the number of increments
  num_inc = 10;

  // Loop to output status in increments
  //  - Close and open the file repeatedly to make it available to the outside
  fprintf(fp_status, "0% ");
  fclose(fp_status);
  fp_status = fopen("status.txt", "a");
  for (i=1; i<=num_inc; i++)
  {
    hold(run_time / num_inc);
    fprintf(fp_status, "%d ", (i * num_inc));
    fclose(fp_status);
    fp_status = fopen("status.txt", "a");
  }
}
