//========================================================= file = mrun.c =====
//=  Run function for REMOTE master                                           =
//=============================================================================
//=  Notes: 1)                                                                =
//=---------------------------------------------------------------------------=
//=  Contains: mrun() - Runs programs on remote hosts                         =
//=---------------------------------------------------------------------------=
//=  History:  KJC (09/19/01) - Genesis                                       =
//=============================================================================
#define EXTERNAL               // This is an external file (used in master.h)

//----- Include files ---------------------------------------------------------
#include "master.h"            // Needed for master stuff
#include "common.h"            // Needed for common stuff

//=============================================================================
//= Function to run programs on remote hosts                                  =
//=============================================================================
int mrun(void)
{
  FILE          *st1;                      // File stream #1
  FILE          *st2;                      // File stream #2
  char          host_name[80];             // Remote host name
  char          exe_file[80];              // Executable file name
  char          in_file[80];               // Input file name
  char          out_file[80];              // Output file name
  unsigned char bin_buf[BUF_SIZE];         // Binary buffer
  int           i;                         // Loop counter
  int           retcode;                   // Return code
  int           done_flag;                 // Done flag for job list
  int           assign_flag;               // Assignment flag

  // Output banner
  get_time(Time);
  printf("Starting master for remote runs at %s... \n", Time);
  printf("  >>> Ctrl-C to break-out <<< \n");

  // Kick-off thread
  retcode = _beginthread(mrun_thread, 4096, NULL);
  if (retcode < 0)
  {
    printf("*** ERROR in mrun() - Unable to create thread \n");
    return(-1);
  }

  // Open job_list
  st1 = fopen(JOB_LIST, "r");
  if (st1 == NULL)
  {
     printf("*** ERROR in mrun() - unable to open job list %s \n", JOB_LIST);
     return(-1);
  }

  // Open host_list
  st2 = fopen(HOST_LIST, "r");
  if (st2 == NULL)
  {
     printf("*** ERROR in mrun() - unable to open host list %s \n", HOST_LIST);
     return(-1);
  }

  // Build the run list
  Num_jobs = 0;
  while(TRUE)
  {
    fscanf(st1, "%s %s %s", exe_file, in_file, out_file);
    if (feof(st1)) break;
    strcpy(Run_list[Num_jobs].exe_file, exe_file);
    strcpy(Run_list[Num_jobs].in_file, in_file);
    strcpy(Run_list[Num_jobs].out_file, out_file);
    if (!feof(st2))
      fscanf(st2, "%s", host_name);
    if (feof(st2))
      strcpy(host_name, "");
    strcpy(Run_list[Num_jobs].host_name, host_name);
    if (strcmp(host_name, "") != 0)
      Run_list[Num_jobs].status = ASSIGNED;
    else
      Run_list[Num_jobs].status = UNASSIGNED;
    Num_jobs++;
  }
  fclose(st1);
  fclose(st2);

  // Run the jobs
  done_flag = FALSE;
  while(done_flag == FALSE)
  {
    // Test if an assignment can be made
    assign_flag = FALSE;
    for (i=0; i<Num_jobs; i++)
      if ((strcmp(Run_list[i].host_name, "") != 0) &&
          (Run_list[i].status == ASSIGNED))
      {
        strcpy(exe_file, Run_list[i].exe_file);
        strcpy(in_file, Run_list[i].in_file);
        strcpy(out_file, Run_list[i].out_file);
        strcpy(host_name, Run_list[i].host_name);
        Run_list[i].status = RUNNING;
        assign_flag = TRUE;
        break;
      }

    // Make an assignment if it can be done
    if (assign_flag == TRUE)
    {
      // Output a job assigned banner
      get_time(Time);
      printf("Running (%s %s %s) on %s at %s \n", exe_file, in_file, out_file,
        host_name, Time);

      // Send the exe file
      retcode = send_file(exe_file, host_name, RUN_PORT1);
      if (retcode < 0)
      {
        printf("*** ERROR in mrun() - send_file() failed \n");
        return(-1);
      }

      // Sleep for turn-around
      Sleep(TURN_DELAY);

      // Send the input file
      retcode = send_file(in_file, host_name, RUN_PORT1);
      if (retcode < 0)
      {
        printf("*** ERROR in mrun() - send_file() failed \n");
        return(-1);
      }

      // Sleep for turn-around
      Sleep(TURN_DELAY);

      // Create a dummy file for the output file
      st1 = fopen(out_file, "w");
      if (st1 == NULL)
      {
        printf("*** ERROR in mrun() - unable to open dummy output file %s \n",
          out_file);
        return(-1);
      }

      // Put message in dummy output file and then close the file
      fprintf(st1, "Dummy");
      fclose(st1);

      // Send a "dummy" for the output file
      retcode = send_file(out_file, host_name, RUN_PORT1);
      if (retcode < 0)
      {
        printf("*** ERROR in mrun() - send_file() failed \n");
        return(-1);
      }
    }

    // Sleep for RUN_DELAY milliseconds
    Sleep(RUN_DELAY);

    // Test if all done
    done_flag = TRUE;
    for (i=0; i<Num_jobs; i++)
      if (Run_list[i].status != DONE) done_flag = FALSE;
  }

  return(1);
}

//===========================================================================
//=  Thread code for mrun()                                                 =
//===========================================================================
void mrun_thread(void *thread_arg)
{
  char         host_name[80];              // Remote host name
  char         out_file[80];               // Output file name
  char         avail_host[80];             // Available host name
  int          i;                          // Loop counter
  int          retcode;                    // Return code

  while(TRUE)
  {
    // Receive the output file
    retcode = recv_file(out_file, host_name, RUN_PORT2);
    if (retcode < 0)
    {
      printf("*** ERROR in mrun_thread() - recv_file() failed \n");
      _endthread();
    }

    // Output a done message
    get_time(Time);
    printf("Job with outfile %s completed at %s \n", out_file, Time);

    // Set status to DONE and assign host_name to available_host
    for (i=0; i<Num_jobs; i++)
      if (strcmp(Run_list[i].out_file, out_file) == 0)
      {
        Run_list[i].status = DONE;
        strcpy(avail_host, Run_list[i].host_name);
        strcpy(Run_list[i].host_name, "");
        break;
      }

    // Assign the available_host to first unassigned job if there are any
    for (i=0; i<Num_jobs; i++)
      if ((strcmp(Run_list[i].host_name, "") == 0) &&
          (Run_list[i].status == UNASSIGNED))
      {
        strcpy(Run_list[i].host_name, avail_host);
        Run_list[i].status = ASSIGNED;
        break;
      }
  }

  _endthread();
}
